@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2009 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.298 2010/03/18 08:09:38 lav Exp $ */
+/* $Id: commands.cc,v 1.300 2010/06/10 09:02:48 lav Exp $ */
 
 #include <config.h>
 
@@ -424,8 +424,11 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"repeat",  cmd_repeat, N_("repeat [OPTS] [delay] [command]"),
 	 N_("Repeat specified command with a delay between iterations.\n"
 	 "Default delay is one second, default command is empty.\n"
-	 " -c <count>  number of iterations\n"
-	 " -d <delay>  delay between iterations.\n")},
+	 " -c <count>  maximum number of iterations\n"
+	 " -d <delay>  delay between iterations\n"
+	 " --while-ok  stop when command exits with non-zero code\n"
+	 " --until-ok  stop when command exits with zero code\n"
+	 " --weak      stop when lftp moves to background.\n")},
    {"reput",   cmd_get,    0,
 	 N_("Usage: reput <lfile> [-o <rfile>]\n"
 	 "Same as `put -c'\n")},
@@ -2584,7 +2587,7 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2009);
+      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2010);
    printf("\n");
    printf(
       _("LFTP is free software: you can redistribute it and/or modify\n"
