@@ -2,8 +2,11 @@
 #include <unistd.h>
 #include "CmdExec.h"
 
-int main()
+char *program_name;
+
+int main(int argc,char **argv)
 {
+   program_name=argv[0];
    CmdExec exec(0,0);
    exec.FeedCmd("open ftp://ftp.yar.ru/pub/source/lftp/\n");
    exec.FeedCmd("cls -l\n");
